<div role="main">
	<div class="main hero">
		<div class="container">
			<div class="row row1">
				<div class="col-md-6 col1">
					{% block search %}
						{% snippet 'home/snippets/search.html' %}
					{% endblock %}
				</div>

				<!-- dtheiler start -->
				<div class="col-md-6 col1">
					{% block recomm_homepage %}
						{% snippet 'home/snippets/recomm_homepage.html' %}
					{% endblock %}
				</div>
				<!-- dtheiler end -->

				<!-- dtheiler start  -->
				<!--<div class="col-md-6 col1"> -->
					{#
						{% block promoted %}
							{% snippet 'home/snippets/promoted.html' %}
						{% endblock %}
					#}
				<!--</div>-->
				<!-- dtheiler end -->
			</div>
		</div>
	
	    <!--dtheiler start 
		moved "container" up to this div
		<div class="main">
		dtheiler end-->
		<div class="container">
			<div class="row row2">
				<div class="col-md-6 col1">
					{# Note: this featured_group block is used as an example in the theming
						tutorial in the docs! If you change this code, be sure to check
						whether you need to update the docs. #}
					{# Start template block example. #}
					{% block featured_group %}
						{% snippet 'home/snippets/featured_group.html' %}
					{% endblock %}
					{# End template block example. #}
				</div>
				<div class="col-md-6 col3">
					{% block featured_organization %}
						{% snippet 'home/snippets/featured_organization.html' %}
					{% endblock %}
				</div>
			</div>
		</div>
		<!--dtheiler start 
		</div>
		dtheiler end-->
	</div>	
</div>