{% ckan_extends %}
{% block page_primary_action %}
  {% if h.check_access('package_create', {'owner_org': group_dict.id}) %}
    {% snippet 'snippets/add_dataset.html', group=group_dict.id %}
    {% snippet 'snippets/add_dataset.html', group=group_dict.id, dataset_type='application' %}
    {% snippet 'snippets/add_dataset.html', group=group_dict.id, dataset_type='service' %}
{#
      {% link_for _('Push to Central Node'), named_route='ids_actions.push_organization_view', id=group_dict.id, class_='btn btn-success', icon='layer-forward' %}
#}
  {% endif %}
{% endblock %}