{%  ckan_extends %}

{% block page_primary_action %}
    {% if h.check_access('organization_create') %}
        {% link_for h.humanize_entity_type('organization', org_type, 'add link') or _('Add Organization'), named_route=org_type ~ '.new', class_="btn btn-primary", icon="plus-square" %}
    {% endif %}
{% endblock %}