{% ckan_extends %}
            {% block header_site_navigation_tabs %}

		          {{ h.build_nav_main(
		            ('dataset.search', _('Datasets'), ["dataset", "resource"]),
                ('service.search', _('Services'), ["service", "resource"]),
                ('application.search', _('Applications'), ["application", "resource"]),
		            ('home.about', _('About')) ) }}
	          {% endblock %}
